import os
import pytest

from ai_dashboard.api.client import Client


TEST_TOKEN = os.getenv("TEST_TOKEN")


@pytest.fixture(scope="session")
def test_token() -> str:
    return TEST_TOKEN


@pytest.fixture(scope="session")
def test_client(test_token: str) -> Client:
    return Client(test_token)



import time

from typing import Optional, Dict, Any

from ai_dashboard.api import endpoints, helpers
from ai_dashboard.dashboard import dashboard

from ai_dashboard import constants
from ai_dashboard import errors


class Client:
    def __init__(self, token: str) -> None:

        self._credentials = helpers.process_token(token)
        self._token = token
        self._endpoints = endpoints.Endpoints(credentials=self._credentials)

        try:
            response = self.list_deployables()
        except:
            raise errors.AuthException
        else:
            self._deployables = sorted(
                response["deployables"], key=lambda x: x["insert_date_"]
            )
            print(constants.WELCOME_MESSAGE.format(self._credentials.project))

    @property
    def deployables(self):
        if not hasattr(self, "_deployables"):
            response = self.list_deployables()
            self._deployables = sorted(
                response["deployables"], key=lambda x: x["insert_date_"]
            )
        return self._deployables

    def recent(self) -> dashboard.Dashboard:
        configuration = self.deployables[-1]
        dataset_id = configuration.get("dataset_id")
        deployable_id = configuration.get("deployable_id")
        project_id = configuration.get("project_id")
        return dashboard.Dashboard(
            endpoints=self._endpoints,
            dataset_id=dataset_id,
            deployable_id=deployable_id,
            configuration=configuration,
            project_id=project_id,
        )

    def list_deployables(self, page_size: int = 1000):
        return self._endpoints._list_deployables(page_size=page_size)

    def create_deployable(
        self,
        dataset_id: Optional[str] = None,
        deployable_id: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        name: Optional[str] = None,
    ) -> Dict[str, Any]:
        if deployable_id is None:
            if name is not None:
                configs = [
                    d
                    for d in self.deployables
                    if d["configuration"].get("deployable_name", "") == name
                    and d.get("dataset_id") == dataset_id
                ]
                try:
                    deployable_id = configs[0]["deployable_id"]
                except:
                    pass
                else:
                    return self._endpoints._get_deployable(
                        deployable_id=deployable_id,
                    )

            if config is None:
                config = {
                    "private": False,
                    "tabs": {},
                    "filters": {},
                    "dataset_name": dataset_id,
                    "type": "explorer",
                    "filterToggles": {},
                    "deployable_name": name,
                    "cacheTimestamp": int(time.time() * 1000),
                    "api_key": "",
                    "project_id": self._credentials.project,
                    "tabOrder": [],
                    "deployable_id": "",
                    "read_key": "",
                }
            return self._endpoints._create_deployable(
                dataset_id=dataset_id,
                config=config,
            )
        else:
            return self._endpoints._get_deployable(
                deployable_id=deployable_id,
            )

    def delete_deployable(self, deployable_id: str) -> None:
        return self._endpoints._delete_deployable(deployable_id=deployable_id)

    def Dashboard(
        self,
        dataset_id: Optional[str] = None,
        delpoyable_id: Optional[str] = None,
        name: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> dashboard.Dashboard:
        assert (
            dataset_id is not None or delpoyable_id is not None
        ), "Set either `dataset_id` and `config` or `deployable_id`"
        response = self.create_deployable(
            dataset_id=dataset_id,
            deployable_id=delpoyable_id,
            config=config,
            name=name,
        )
        return dashboard.Dashboard(endpoints=self._endpoints, **response)



import requests

from ai_dashboard import __version__
from ai_dashboard.types import Credentials

from typing import Optional, Dict, Any, List


class Endpoints:
    def __init__(self, credentials: Credentials) -> None:
        self._credentials = credentials
        self._base_url = (
            f"https://api-{self._credentials.region}.stack.tryrelevance.com/latest"
        )
        self._headers = dict(
            Authorization=f"{self._credentials.project}:{self._credentials.api_key}",
        )

    def _create_deployable(
        self, dataset_id: Optional[str] = None, config: Optional[Dict[str, Any]] = None
    ):
        return requests.post(
            url=self._base_url + "/deployables/create",
            headers=self._headers,
            json=dict(
                dataset_id=dataset_id,
                configuration={} if config is None else config,
            ),
        ).json()

    def _share_deployable(self, deployable_id: str):
        return requests.post(
            url=self._base_url + f"/deployables/{deployable_id}/share",
            headers=self._headers,
        ).json()

    def _unshare_deployable(self, deployable_id: str):
        return requests.post(
            url=self._base_url + f"/deployables/{deployable_id}/private",
            headers=self._headers,
        ).json()

    def _update_deployable(
        self,
        deployable_id: str,
        dataset_id: str,
        configuration: Optional[Dict] = None,
        overwrite: bool = True,
        upsert: bool = True,
    ):
        return requests.post(
            url=self._base_url + f"/deployables/{deployable_id}/update",
            headers=self._headers,
            json=dict(
                dataset_id=dataset_id,
                configuration=configuration,
                overwrite=overwrite,
                upsert=upsert,
            ),
        ).json()

    def _share_dashboard(self, deployable_id: str):
        return requests.post(
            url=self._base_url + f"/deployablegroups/{deployable_id}/share",
            headers=self._headers,
        ).json()

    def _unshare_dashboard(self, deployable_id: str):
        return requests.post(
            url=self._base_url + f"/deployablegroups/{deployable_id}/private",
            headers=self._headers,
        ).json()

    def _get_deployable(self, deployable_id: str):
        return requests.get(
            url=self._base_url + f"/deployables/{deployable_id}/get",
            headers=self._headers,
        ).json()

    def _delete_deployable(self, deployable_id: str):
        return requests.post(
            url=self._base_url + f"/deployables/delete",
            headers=self._headers,
            json=dict(
                id=deployable_id,
            ),
        ).json()

    def _list_deployables(self, page_size: int):
        return requests.get(
            url=self._base_url + "/deployables/list",
            headers=self._headers,
            params=dict(
                page_size=page_size,
            ),
        ).json()

    def _get_file_upload_urls(self, dataset_id: str, files: List[str]):
        return requests.post(
            url=self._base_url + f"/datasets/{dataset_id}/get_file_upload_urls",
            headers=self._headers,
            json=dict(files=files),
        ).json()

    def _upload_media(self, presigned_url: str, media_content: bytes):
        # dont use .json() since response cannot be json decoded
        return requests.put(presigned_url, data=media_content)



from typing import Optional, Dict, Any

from ai_dashboard.api import endpoints
from ai_dashboard.tabs import abstract_tab


class CategoryView(abstract_tab.Tab):
    ID = "CLUSTER_VIEW"

    BLANK: Dict[str, Any] = {
        "activeFilterGroup": "",
        "color": None,
        "configuration": {},
        "name": "",
        "type": ID,
    }

    def __init__(
        self,
        endpoints: endpoints.Endpoints,
        dataset_id: str,
        title: Optional[str] = None,
        name: Optional[str] = None,
        colour: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        super().__init__()

        self._endpoints = endpoints

        if config is not None:
            self._config = config
        else:
            self.reset()

        self.dataset_id = dataset_id
        self.config["colour"] = colour
        self.config["configuration"]["title"] = self.ID if title is None else title
        self.config["name"] = name

    @classmethod
    def from_fields(
        cls,
        primary_field: str,
        secondary_field: str,
        cluster_field: str = "userImage",
        subcluster_field: str = "",
        layout_template: str = "custom",
        title: Optional[str] = None,
        name: Optional[str] = None,
        colour: Optional[str] = None,
    ):
        category_view = cls(name=name, title=title, colour=colour)
        category_view.config = {
            "activeFilterGroup": "",
            "color": colour,
            "configuration": {
                "previewDocumentCardConfiguration": {
                    "layoutTemplateConfiguration": {
                        "fields": [
                            {
                                "id": "primary",
                                "value": primary_field,
                            },
                            {
                                "id": "secondary",
                                "value": secondary_field,
                            },
                        ]
                    },
                    "layoutTemplate": layout_template,
                },
                "subclusterField": subcluster_field,
                "clusterField": cluster_field,
            },
            "name": name,
            "type": cls.ID,
        }
        return category_view

    def set_view(
        self,
        primary_field: str,
        secondary_field: str = None,
        cluster_field: str = "userImage",
        subcluster_field: str = "",
        layout_template: str = "custom",
    ):
        self.config["configuration"] = {
            "previewDocumentCardConfiguration": {
                "layoutTemplateConfiguration": {"fields": []},
                "layoutTemplate": layout_template,
            },
            "subclusterField": subcluster_field,
            "clusterField": cluster_field,
        }
        if primary_field is not None:
            self.config["configuration"]["previewDocumentCardConfiguration"][
                "layoutTemplateConfiguration"
            ]["fields"].append(
                {
                    "id": "primary",
                    "value": primary_field,
                }
            )
        if secondary_field is not None:
            self.config["configuration"]["previewDocumentCardConfiguration"][
                "layoutTemplateConfiguration"
            ]["fields"].append(
                {
                    "id": "secondary",
                    "value": secondary_field,
                }
            )



from typing import Optional, List, Dict, Any


def aggregation_chart(
    groupby: List[Dict[str, Any]],
    metric: List[Dict[str, Any]],
    chart_type: str = "column",
    title: Optional[str] = None,
    page_size: int = 20,
    show_frequencies: bool = False,
    sentiment_field: str = "",
    y_axis_sort_field: str = "",
    sort_direction: str = "desc",
    filters: Optional[List[Dict[str, Any]]] = None,
):
    """
    Example for groupby
    [
        {
            "agg": "max",
            "field": "score",
            "color": "",
            "name": "max thumbsUpCount",
            "lowerIsBetter": false
        }
    ]
    Example for metric
    [
        {
            "agg": "category",
            "field": "content"
        }
    ]
    """
    assert sort_direction in {"desc", "asc"}

    for query in groupby:
        query["aggType"] = "groupby"

    for query in metric:
        query["aggType"] = "metric"

    if filters is None:
        filters = []

    return [
        {
            "type": "appBlock",
            "content": [
                {
                    "type": "datasetAggregation",
                    "attrs": {
                        "uid": "",
                        "title": title,
                        "chartType": chart_type,
                        "filters": filters,
                        "xAxis": {
                            "fields": groupby,
                            "numResults": page_size,
                            "resortAlphanumerically": False,
                        },
                        "yAxis": {
                            "fields": metric,
                            "showFrequency": show_frequencies,
                            "sortBy": y_axis_sort_field,
                            "sortDirection": sort_direction,
                        },
                        "timeseries": {
                            "field": "insert_date_",
                            "interval": "monthly",
                        },
                        "sentiment": {
                            "field": sentiment_field,
                            "mode": "overview",
                            "interval": "monthly",
                        },
                        "wordCloud": {"mode": "cloud"},
                    },
                }
            ],
        },
    ]
